// (c) 2016, SAS Institute Inc.

jQuery.sap.declare("sas.risk.modimp.implementation.wizard.widget.HorizonInput");

jQuery.sap.require("sas.hc.m.Input");
jQuery.sap.require("sas.hc.m.Label");

jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");

jQuery.sap.require("sas.risk.modimp.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.modimp.implementation.utils.ModelImplementationConstants");
jQuery.sap.require("sas.risk.modimp.implementation.utils.ModelImplementationUtil");


/**
 * Provides the horizons input along with all associated validation and logic for
 * any wizard step that wants to consume it. Provides a createContent function
 * that produces the DOM content for this entity. The controller must be passed
 * to the constructor and define a public function named handleInputChanged to
 * react to changes.
 *
 * @name sas.risk.modimp.implementation.wizard.widgets.HorizonInput
 */
sap.ui.base.Object.extend("sas.risk.modimp.implementation.wizard.widgets.HorizonInput", {

    /**
     * Constructs a new instance of HorizonInput with the specified controller.
     * @param {Object} controller  controller for the view consuming this input
     * @param {String} width  the desired width of the control
     */
    constructor: function (controller, width) {
        var self = this;

        self.myController = controller;

        self.MIPCommonUtil = sas.risk.modimp.common.utils.CommonUtil;
        self.ModelImplementationConstants = sas.risk.modimp.implementation.utils.ModelImplementationConstants;
        self.ModelImplementationUtil = sas.risk.modimp.implementation.utils.ModelImplementationUtil;

        self.rb = self.MIPCommonUtil.getBundle();

        self.horizonInput = new sas.hc.m.Input({
            width: width,
            type: sap.m.InputType.Number,
            value: "{/runInstanceConfig/horizon}",
            disableValidation: false,  // despite the name, false actually disables default validation
            enableGroupingSeparator: false,
            selectTextOnPress: false,
            valueLiveUpdate: true,
            maxLength: 10,
            liveChange: $.proxy(self._handleLiveChange, self)
        });
        /*
         * FIXME this is a hack for MIP 2.1/HC24-25
         * The commons input component calls an internal function _performValidation when
         * focus is removed (see sas.hc.m.Input.prototype.onfocusout). This replaces the
         * value state and value state text with whatever the component validation resolves
         * to. The component validation is by field type, in our case number, so as long as
         * the input value is a valid number, when focus is removed the fields is restored
         * to a valid state. Recommend HC provide a way to specify custom validation logic
         * to run along with or in place of the current internal validation scheme.
         *
         * Defect: S1195414
         */
        self.horizonInput._performValidation = function(){};    // disable commons validation that removes our validation
        self.horizonInput._performLiveValidation = function(){};    // disable commons validation that removes our validation
    }
});


// internal functions

sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype._handleLiveChange = function(event) {
    var self = this;
    var liveValue = self.horizonInput.getValue();

    self.currentLiveChangeEventTime = new Date().getTime();

    if(!liveValue) {
        // no value, so no error
        self.horizonInput.setValueState(sap.ui.core.ValueState.None);
        self.horizonInput.setValueStateText(null);

        // notify controller of possible input change
        self.myController.handleInputChanged();
    }
    else if(self.ModelImplementationUtil.isValidHorizonsValue(liveValue)) {
        // run custom validation only if the consumer supplied a function
        if(typeof self.customValidationFunction === "function") {
            // apply custom validation, but only after the user has 'finished' typing
            var previousLiveChangeEventTime = self.currentLiveChangeEventTime;
            setTimeout(function () {
                if(previousLiveChangeEventTime === self.currentLiveChangeEventTime) {
                    // run custom validation
                    self.customValidationFunction();

                    // notify controller of possible input change
                    self.myController.handleInputChanged();
                }
            }, 750);
        }
        else {
            // set the input value state
            self.horizonInput.setValueState(sap.ui.core.ValueState.None);
            self.horizonInput.setValueStateText(null);

            // notify controller of input change
            self.myController.handleInputChanged();
        }
    }
    else {
        // set the input value state
        self.horizonInput.setValueState(sap.ui.core.ValueState.Error);
        self.horizonInput.setValueStateText("Specify an integer from 1 to 2147483647.");

        // notify controller of input change
        self.myController.handleInputChanged();
    }
};


// public functions

/**
 * Set a custom validation function and error message. The validation function is responsible
 * for setting the value state on the input based on the validation results.
 *
 * @param {Function} validationFunction
 */
sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype.setCustomValidationFunction = function(validationFunction) {
    this.customValidationFunction = validationFunction;
};

/**
 * Attaches the specified function to the liveChange event of the input.
 * @param {Function} listenerFunction
 */
sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype.attachLiveChange = function(listenerFunction) {
    this.horizonInput.attachLiveChange(listenerFunction);
};

/**
 * Attaches the specified function to the change event of the input.
 * @param {Function} listenerFunction
 */
sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype.attachChange = function(listenerFunction) {
    this.horizonInput.attachChange(listenerFunction);
};

/**
 * Sets the value state of the horizon input.
 * @returns {sap.ui.core.ValueState}
 */
sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype.setValueState = function(valueState, valueStateText) {
    this.horizonInput.setValueState(valueState);
    this.horizonInput.setValueStateText(valueStateText);
};

/**
 * Returns the value state of the horizon input.
 * @returns {sap.ui.core.ValueState}
 */
sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype.getValueState = function() {
    return this.horizonInput.getValueState();
};

/**
 * Returns the value of the horizon input.
 * @returns {String}
 */
sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype.getValue = function() {
    return this.horizonInput.getValue();
};

///**
// * Returns true if the entered horizon value is valid, false otherwise.
// * @returns {Boolean|boolean}
// */
//sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype.isValidHorizonsValue = function(horizons) {
//    return sas.risk.common.utils.CommonUtil.isPositiveInteger(horizons) && parseInt(horizons) <= sas.risk.common.utils.CommonConstants.MAX_HORIZON.MAX_HORIZON;
//};

/**
 * Creates and returns the content for the horizon input.
 */
sas.risk.modimp.implementation.wizard.widgets.HorizonInput.prototype.createContent = function() {
    var horizonsSelectorLabel = new sas.hc.m.Label({
        text: this.rb.getText("ModelImplementation.RunInstanceInput.HorizonsInput.txt"),
        required: true
    });
    horizonsSelectorLabel.addStyleClass("mimInputLabel");
    horizonsSelectorLabel.setLabelFor(this.horizonInput);

    return new sas.hc.ui.layout.VerticalLayout({
        content: [horizonsSelectorLabel, this.horizonInput]
    }).addStyleClass("mipWizardInputLayout");
};